@@ -28,6 +28,7 @@ from modules.utils.copy import safe_copy
 from modules.utils.ids import generate_event_id, get_session_id, read_app_uid, get_project_mcpower_dir
 from modules.utils.json import safe_json_dumps, to_dict
 from modules.utils.mcp_configs import extract_wrapped_server_info
+from modules.utils.string import truncate_at
 from wrapper.schema import merge_input_schema_with_existing
 
 
@@ -173,7 +174,7 @@ class SecurityMiddleware(Middleware):
     async def secure_elicitation_handler(self, message, response_type, params, context):
         # FIXME: elicitation message, params, and context should be redacted before logging
         self.logger.info(f"secure_elicitation_handler: "
-                         f"message={str(message)[:100]}..., response_type={response_type},"
+                         f"message={truncate_at(str(message), 100)}, response_type={response_type},"
                          f"params={params}, context={context}")
 
         mock_context = MockContext(
@@ -203,7 +204,7 @@ class SecurityMiddleware(Middleware):
 
     async def secure_log_handler(self, log_message):
         # FIXME: log_message should be redacted before logging,
-        self.logger.info(f"secure_log_handler: {str(log_message)[:100]}...")
+        self.logger.info(f"secure_log_handler: {truncate_at(str(log_message), 100)}")
         # FIXME: log_message should be reviewed with policy before forwarding
 
         # Handle case where log_message.data is a string instead of dict
@@ -568,9 +569,13 @@ class SecurityMiddleware(Middleware):
             base_dict = await self._build_baseline_policy_dict(event_id, context, wrapper_args, tool_args)
             policy_request = create_policy_request(
                 event_id=event_id,
-                server_name=base_dict["server"]["name"],
-                server_transport=base_dict["server"]["transport"],
-                tool_name=base_dict["tool"]["name"] or base_dict["tool"]["method"],
+                server=ServerRef(
+                    name=base_dict["server"]["name"],
+                    transport=base_dict["server"]["transport"]
+                ),
+                tool=ToolRef(
+                    name=base_dict["tool"]["name"] or base_dict["tool"]["method"]
+                ),
                 agent_context=base_dict["agent_context"],
                 env_context=base_dict["environment_context"],
                 arguments=tool_args,
@@ -596,9 +601,13 @@ class SecurityMiddleware(Middleware):
             base_dict = await self._build_baseline_policy_dict(event_id, context, wrapper_args, tool_args)
             policy_response = create_policy_response(
                 event_id=event_id,
-                server_name=base_dict["server"]["name"],
-                server_transport=base_dict["server"]["transport"],
-                tool_name=base_dict["tool"]["name"] or base_dict["tool"]["method"],
+                server=ServerRef(
+                    name=base_dict["server"]["name"],
+                    transport=base_dict["server"]["transport"]
+                ),
+                tool=ToolRef(
+                    name=base_dict["tool"]["name"] or base_dict["tool"]["method"]
+                ),
                 response_content=safe_json_dumps(result),
                 agent_context=base_dict["agent_context"],
                 env_context=base_dict["environment_context"],
